@@ -1,6 +1,6 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
-# Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt.
 # 
@@ -33,10 +33,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040901
-pyqt_version_str = "4.9.1"
+pyqt_version = 0x040906
+pyqt_version_str = "4.9.6"
 
-sip_min_version = 0x040d01
+sip_min_version = 0x040e02
 
 qt_version = 0
 qt_edition = ""
@@ -51,6 +51,10 @@ qt_xfeatures = None
 qt_shared = ""
 qt_framework = 0
 
+# This default value will be used by the code that bootstraps the Qt
+# configuration.
+qt_macx_spec = 'macx-g++'
+
 qt_sip_flags = []
 
 pyqt_modules = []
@@ -70,9 +74,9 @@ dbuslibs = []
 # dynamic linker won't be able to resolve the symbols.  On other systems we
 # assume we can just run qmake by using its full pathname.
 if sys.platform == 'win32':
-    MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH."
+    MSG_CHECK_QMAKE = "Make sure you have a working Qt qmake on your PATH."
 else:
-    MSG_CHECK_QMAKE = "Make sure you have a working Qt v4 qmake on your PATH or use the -q argument to explicitly specify a working Qt v4 qmake."
+    MSG_CHECK_QMAKE = "Make sure you have a working Qt qmake on your PATH or use the -q argument to explicitly specify a working Qt qmake."
 
 
 def find_default_qmake():
@@ -173,9 +177,12 @@ def create_optparser():
     g.add_option("-T", "--no-timestamp", action="store_true", default=False,
             dest="no_timestamp", help="suppress timestamps in the header "
             "comments of generated code [default: include timestamps]")
+    g.add_option("--no-deprecated", action="store_false", default=True,
+            dest="with_deprecated", help="disable Qt v4 features deprecated "
+            "in Qt v5 [default: enabled]")
 
     if sys.platform != 'win32':
-        if sys.platform in ('linux2', 'linux3', 'darwin'):
+        if sys.platform.startswith('linux') or sys.platform == 'darwin':
             pip_default = True
             pip_default_str = "enabled"
         else:
@@ -335,7 +342,10 @@ class ConfigurePyQt4:
             0x040701: "Qt_4_7_0",
             0x040702: "Qt_4_7_1",
             0x040800: "Qt_4_7_2",
-            0x050000: "Qt_4_8_0"
+            0x040803: "Qt_4_8_0",
+            0x040804: "Qt_4_8_3",
+            0x050000: "Qt_4_8_4",
+            0x060000: "Qt_5_0_0"
         }
 
     def check_modules(self):
@@ -357,25 +367,30 @@ class ConfigurePyQt4:
 
         check_module("QtGui", "qwidget.h", "new QWidget()")
         check_module("QtHelp", "qhelpengine.h", "new QHelpEngine(\"foo\")")
-        check_module("QtMultimedia", "QAudioDeviceInfo",
+        if qt_version < 0x050000: check_module("QtMultimedia", "QAudioDeviceInfo",
                 "new QAudioDeviceInfo()")
         check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
-        check_module("QtDBus", "qdbusconnection.h",
+        if qt_version < 0x050000: check_module("QtDBus", "qdbusconnection.h",
                 "QDBusConnection::systemBus()")
-        check_module("QtDeclarative", "qdeclarativeview.h",
+        if qt_version < 0x050000: check_module("QtDeclarative", "qdeclarativeview.h",
                 "new QDeclarativeView()")
-        check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
-        check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
-        check_module("QtScriptTools", "qscriptenginedebugger.h",
+        if qt_version < 0x050000: check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
+        if qt_version < 0x050000: check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
+        if qt_version < 0x050000: check_module("QtScriptTools", "qscriptenginedebugger.h",
                 "new QScriptEngineDebugger()")
         check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()",
                 extra_libs=sql_libs)
         check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
-        check_module("QtTest", "QtTest", "QTest::qSleep(0)")
+        if qt_version < 0x050000: check_module("QtTest", "QtTest", "QTest::qSleep(0)")
         check_module("QtWebKit", "qwebpage.h", "new QWebPage()")
         check_module("QtXml", "qdom.h", "new QDomDocument()")
-        check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
-        check_module("phonon", "phonon/videowidget.h",
+
+        # Qt v5-beta1 causes compiler error messages.  Wait to see if it fixed
+        # in a later release.
+        if qt_version < 0x050000:
+            check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
+
+        if qt_version < 0x050000: check_module("phonon", "phonon/videowidget.h",
                 "new Phonon::VideoWidget()")
         check_module("QtAssistant", "qassistantclient.h",
                 "new QAssistantClient(\"foo\")", extra_lib_dirs=ass_lib_dirs,
@@ -386,10 +401,10 @@ class ConfigurePyQt4:
         elif sipcfg.universal:
             sipconfig.inform("QtDesigner module disabled with universal binaries.")
         else:
-            check_module("QtDesigner", "QExtensionFactory",
+            if qt_version < 0x050000: check_module("QtDesigner", "QExtensionFactory",
                     "new QExtensionFactory()")
 
-        check_module("QAxContainer", "qaxobject.h", "new QAxObject()",
+        if qt_version < 0x050000: check_module("QAxContainer", "qaxobject.h", "new QAxObject()",
                 extra_libs=["QAxContainer"])
 
         if os.path.isdir(os.path.join(src_dir, "dbus")):
@@ -804,6 +819,9 @@ class ConfigurePyQt4:
                 if sipcfg.sip_inc_dir != sipcfg.py_inc_dir:
                     inc_path.insert(0, sipcfg.sip_inc_dir)
 
+                if sipcfg.py_conf_inc_dir != sipcfg.py_inc_dir:
+                    inc_path.insert(0, sipcfg.py_conf_inc_dir)
+
                 if opts.bigqt:
                     api_dir = "../../_qt"
                 else:
@@ -921,6 +939,8 @@ include(%s)
         if "QtXml" in pyqt_modules:
             sipconfig.inform("Creating pylupdate4 Makefile...")
 
+            cxxflags_app = sipcfg.build_macros().get("CXXFLAGS_APP", "")
+
             makefile = sipconfig.ProgramMakefile(
                 configuration=sipcfg,
                 build_file=os.path.join(src_dir, "pylupdate", "pylupdate.sbf"),
@@ -935,13 +955,22 @@ include(%s)
                 deployment_target=sipcfg.deployment_target
             )
 
-            makefile.extra_include_dirs.append(os.path.join(src_dir, "pylupdate"))
+            makefile.extra_include_dirs.append(
+                    os.path.join(src_dir, "pylupdate"))
+
+            if cxxflags_app != "":
+                makefile.extra_cxxflags.append(cxxflags_app)
+
             makefile.generate()
             tool.append("pylupdate")
 
             sipconfig.inform("Creating pyrcc4 Makefile...")
 
             makefile = pyrccMakefile()
+
+            if cxxflags_app != "":
+                makefile.extra_cxxflags.append(cxxflags_app)
+
             makefile.generate()
             tool.append("pyrcc")
         else:
@@ -963,17 +992,25 @@ include(%s)
                 from distutils.sysconfig import get_config_vars
                 ducfg = get_config_vars()
 
+                config_args = ducfg.get("CONFIG_ARGS", "")
+
                 if sys.platform == "darwin":
-                    # We need to work out how to specify the right framework
-                    # version.
-                    link = "-framework Python"
-                elif "--enable-shared" in ducfg.get("CONFIG_ARGS", ""):
+                    dynamic_pylib = "--enable-framework" in config_args
+
+                    # It's probably a Python bug that the library name doesn't
+                    # include the ABI information.
+                    abi = ""
+                else:
+                    dynamic_pylib = "--enable-shared" in config_args
+
+                if dynamic_pylib:
                     if glob.glob("%s/lib/libpython%d.%d*" % (ducfg["exec_prefix"], py_major, py_minor)):
                         lib_dir_flag = quote("-L%s/lib" % ducfg["exec_prefix"])
                     elif glob.glob("%s/libpython%d.%d*" % (ducfg["LIBDIR"], py_major, py_minor)):
                         lib_dir_flag = quote("-L%s" % ducfg["LIBDIR"])
                     else:
                         sipconfig.inform("Qt Designer plugin disabled because Python library couldn't be found")
+                        lib_dir_flag = ''
                         opts.designer_plugin = False
 
                     link = "%s -lpython%d.%d%s" % (lib_dir_flag, py_major, py_minor, abi)
@@ -1081,6 +1118,11 @@ def inform_user():
     sipconfig.inform("These PyQt modules will be built: %s." % ", ".join(pyqt_modules))
     sipconfig.inform("The PyQt Python package will be installed in %s." % opts.pyqtmoddir)
 
+    if opts.with_deprecated:
+        sipconfig.inform("PyQt is being built with deprecated Qt v4 features")
+    else:
+        sipconfig.inform("PyQt is being built without deprecated Qt v4 features")
+
     if opts.no_docstrings:
         sipconfig.inform("PyQt is being built without generated docstrings.")
     else:
@@ -1261,7 +1303,7 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
     opengl = (mname == "QtOpenGL")
 
     qt = [mname]
-    if mname in ("QtOpenGL", "QtWebKit"):
+    if mname in ("QtAssistant", "QtHelp", "QtOpenGL", "QtWebKit"):
         qt.append("QtCore")
 
     makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0,
@@ -1484,6 +1526,10 @@ def set_sip_flags(pyqt):
         qt_sip_flags.append("-x")
         qt_sip_flags.append("VendorID")
 
+    if not opts.with_deprecated:
+        qt_sip_flags.append("-x")
+        qt_sip_flags.append("PyQt_Deprecated_5_0")
+
     # Handle the platform tag.
     if sys.platform == 'win32':
         plattag = "WS_WIN"
@@ -1726,6 +1772,10 @@ def generate_code(mname, extra_include_dirs=None, extra_lib_dirs=None, extra_lib
 
     add_makefile_extras(makefile, extra_include_dirs, extra_lib_dirs, extra_libs)
 
+    if qt_version >= 0x050000 and opts.with_deprecated:
+        # The name of this macro is very confusing.
+        makefile.extra_defines.append('QT_DISABLE_DEPRECATED_BEFORE=0x040900')
+
     makefile.generate()
 
 
@@ -1850,15 +1900,25 @@ def get_build_macros(overrides):
 
     overrides is a list of macros overrides from the user.
     """
+    global qt_macx_spec
+
     # Get the name of the qmake configuration file to take the macros from.
     if "QMAKESPEC" in list(os.environ.keys()):
         fname = os.environ["QMAKESPEC"]
 
         if not os.path.dirname(fname):
+            qt_macx_spec = fname
             fname = os.path.join(qt_datadir, "mkspecs", fname)
     elif sys.platform == "darwin":
         # The Qt Mac binary installer defaults to xcode which we don't want.
-        fname = os.path.join(qt_datadir, "mkspecs", "macx-g++")
+        # Use Qt5's macx-clang if it is available, otherwise fall back to
+        # macx-g++.
+        fname = os.path.join(qt_datadir, "mkspecs", "macx-clang")
+        if os.path.isdir(fname):
+            qt_macx_spec = "macx-clang"
+        else:
+            fname = os.path.join(qt_datadir, "mkspecs", "macx-g++")
+            qt_macx_spec = "macx-g++"
     else:
         fname = os.path.join(qt_datadir, "mkspecs", "default")
 
@@ -1879,7 +1939,27 @@ def get_build_macros(overrides):
         "QT_INSTALL_LIBS":      qt_libdir
     }
 
-    return sipconfig.parse_build_macros(fname, names, overrides, properties)
+    macros = sipconfig.parse_build_macros(fname, names, overrides, properties)
+
+    if macros is None:
+        return None
+
+    # Qt5 doesn't seem to support the specific macros so add them if they are
+    # missing.
+    if macros.get("INCDIR_QT", "") == "":
+        macros["INCDIR_QT"] = qt_incdir
+
+    if macros.get("LIBDIR_QT", "") == "":
+        macros["LIBDIR_QT"] = qt_libdir
+
+    if macros.get("MOC", "") == "":
+        default_moc = os.path.join(qt_bindir, "moc")
+        if sys.platform == 'win32':
+            default_moc += ".exe"
+
+        macros["MOC"] = default_moc
+
+    return macros
 
 
 def check_qt_installation(macros):
@@ -1928,7 +2008,7 @@ def fix_qmake_args(args=""):
     """
     if sys.platform == "darwin":
         # The Qt binary installer has macx-xcode as the default.
-        args = "-spec %s %s" % (sipcfg.platform, args)
+        args = "-spec %s %s" % (qt_macx_spec, args)
 
     return args
 
@@ -1986,6 +2066,14 @@ SOURCES = %s
 #include <QLibraryInfo>
 #include <QTextStream>
 
+// These seem to be missing from the Qt v5 beta.
+#if !defined(QT_EDITION_DESKTOP)
+#define QT_EDITION_DESKTOP      8
+#endif
+#if !defined(QT_EDITION_OPENSOURCE)
+#define QT_EDITION_OPENSOURCE   8
+#endif
+
 int main(int argc, char **argv)
 {
     QCoreApplication app(argc, argv);
@@ -2064,6 +2152,10 @@ int main(int argc, char **argv)
     out << "PyQt_PrintPreviewWidget\\n";
 #endif
 
+#if defined(QT_NO_RAWFONT)
+    out << "PyQt_RawFont\\n";
+#endif
+
 #if !defined(QT3_SUPPORT) || QT_VERSION >= 0x040200
     out << "PyQt_NoPrintRangeBug\\n";
 #endif
@@ -2110,7 +2202,7 @@ int main(int argc, char **argv)
     run_command(exe_file)
 
     if not os.access(out_file, os.F_OK):
-        sipconfig.error("%s failed to create %s. Make sure your Qt v4 installation is correct." % (exe_file, out_file))
+        sipconfig.error("%s failed to create %s. Make sure your Qt installation is correct." % (exe_file, out_file))
 
     # Read the directories.
     f = open(out_file, "r")
