@@ -182,7 +182,7 @@ class MainWindow(QtGui.QMainWindow):
 
     def closeEvent(self, event):
         self.mdiArea.closeAllSubWindows()
-        if self.activeMdiChild():
+        if self.mdiArea.currentSubWindow():
             event.ignore()
         else:
             self.writeSettings()
@@ -275,7 +275,7 @@ class MainWindow(QtGui.QMainWindow):
 
             action = self.windowMenu.addAction(text)
             action.setCheckable(True)
-            action.setChecked(child == self.activeMdiChild())
+            action.setChecked(child is self.activeMdiChild())
             action.triggered.connect(self.windowMapper.map)
             self.windowMapper.setMapping(action, window)
 
@@ -306,7 +306,8 @@ class MainWindow(QtGui.QMainWindow):
                 statusTip="Save the document under a new name",
                 triggered=self.saveAs)
 
-        self.exitAct = QtGui.QAction("E&xit", self, shortcut="Ctrl+Q",
+        self.exitAct = QtGui.QAction("E&xit", self,
+                shortcut=QtGui.QKeySequence.Quit,
                 statusTip="Exit the application",
                 triggered=QtGui.qApp.closeAllWindows)
 
@@ -325,7 +326,7 @@ class MainWindow(QtGui.QMainWindow):
                 statusTip="Paste the clipboard's contents into the current selection",
                 triggered=self.paste)
 
-        self.closeAct = QtGui.QAction("Cl&ose", self, shortcut="Ctrl+F4",
+        self.closeAct = QtGui.QAction("Cl&ose", self,
                 statusTip="Close the active window",
                 triggered=self.mdiArea.closeActiveSubWindow)
 
