@@ -1,7 +1,7 @@
 // This defines the API provided by this library.  It must not be explicitly
 // included by the library itself.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -52,7 +52,12 @@ int qpycore_pyqtconfigure(PyObject *self, QObject *qobj, PyObject *kwds);
 // Support for converting between PyObject and QString.
 PyObject *qpycore_PyObject_FromQString(const QString &qstr);
 QString qpycore_PyObject_AsQString(PyObject *obj);
+#if !defined(QT_DEPRECATED_SINCE)
+#define QT_DEPRECATED_SINCE(m, n)   1
+#endif
+#if QT_DEPRECATED_SINCE(5, 0)
 const char *qpycore_encode(PyObject **s, QCoreApplication::Encoding encoding);
+#endif
 
 // Support for converting between PyObject and QStringList.
 PyObject *qpycore_PyObject_FromQStringList(const QStringList &qstrlst);
@@ -84,9 +89,11 @@ PyObject *qpycore_qobject_staticmetaobject(PyObject *type_obj);
 // Support for emitting signals.
 bool qpycore_qobject_emit(QObject *qtx, const char *sig, PyObject *sigargs);
 
+#if QT_VERSION < 0x050000
 // Support for QAbstractEventDispatcher.setEventFilter().
 PyObject *qpycore_qabstracteventdispatcher_seteventfilter(
         QAbstractEventDispatcher *disp, PyObject *filter);
+#endif
 
 // Support for QMetaObject.connectSlotsByName().
 void qpycore_qmetaobject_connectslotsbyname(QObject *qobj,
