@@ -1,6 +1,6 @@
 // This is the implementation of the Chimera class.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -412,6 +412,14 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
             // various itemChange() implementations.
             if (_metatype == 0 && is_a_QObject)
             {
+#if QT_VERSION >= 0x050000
+                static int QWidget_metatype = -1;
+
+                // Qt5 seems to register QWidget.
+                if (QWidget_metatype < 0)
+                    QWidget_metatype = QMetaType::type("QWidget*");
+#endif
+
                 PyObject *mro = type_obj->tp_mro;
 
                 Q_ASSERT(PyTuple_Check(mro));
@@ -429,10 +437,15 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
                     // QObjects are always pointers.
                     sc_name.append('*');
 
-                    _metatype = QMetaType::type(sc_name.constData());
+                    int sc_metatype = QMetaType::type(sc_name.constData());
 
-                    if (_metatype >= QMetaType::User)
+#if QT_VERSION >= 0x050000
+                    if (sc_metatype >= QMetaType::User && sc_metatype != QWidget_metatype)
+#else
+                    if (sc_metatype >= QMetaType::User)
+#endif
                     {
+                        _metatype = sc_metatype;
                         _type = sipTypeFromPyTypeObject(sc);
                         _name = sc_name;
 
@@ -448,11 +461,15 @@ bool Chimera::parse_py_type(PyTypeObject *type_obj)
             // about but was registered by Qt.
             if (_metatype < QMetaType::User)
             {
+#if QT_VERSION < 0x050000
                 if (sipType_QWidget && PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QWidget)))
                 {
                     _metatype = QMetaType::QWidgetStar;
                 }
                 else if (is_a_QObject)
+#else
+                if (is_a_QObject)
+#endif
                 {
                     _metatype = QMetaType::QObjectStar;
                 }
@@ -724,11 +741,15 @@ bool Chimera::parse_cpp_type(const QByteArray &type)
         {
             PyTypeObject *type_obj = sipTypeAsPyTypeObject(_type);
 
+#if QT_VERSION < 0x050000
             if (sipType_QWidget && PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QWidget)))
             {
                 _metatype = QMetaType::QWidgetStar;
             }
             else if (PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject)))
+#else
+            if (PyType_IsSubtype(type_obj, sipTypeAsPyTypeObject(sipType_QObject)))
+#endif
             {
                 _metatype = QMetaType::QObjectStar;
             }
@@ -841,6 +862,7 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
                 sipType_QObject, 0, SIP_NO_CONVERTORS, 0, &iserr);
         break;
 
+#if QT_VERSION < 0x050000
     case QMetaType::QWidgetStar:
         if (sipType_QWidget)
         {
@@ -853,6 +875,7 @@ bool Chimera::fromPyObject(PyObject *py, void *cpp) const
         }
 
         break;
+#endif
 
     case QMetaType::QVariantList:
         {
@@ -1153,6 +1176,7 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
                 0, SIP_NO_CONVERTORS, 0, &iserr);
         break;
 
+#if QT_VERSION < 0x050000
     case QMetaType::QWidgetStar:
         if (sipType_QWidget)
         {
@@ -1165,6 +1189,7 @@ bool Chimera::fromPyObject(PyObject *py, QVariant *var, bool strict) const
         }
 
         break;
+#endif
 
     case QMetaType::QVariantList:
         {
@@ -1344,7 +1369,11 @@ PyObject *Chimera::toPyObject(const QVariant &var) const
         // converting it.
         if (_type != sipType_QVariant && _metatype == PyQt_PyObject::metatype)
         {
+#if QT_VERSION >= 0x050000
+            PyQt_PyObject pyobj_wrapper = var.value<PyQt_PyObject>();
+#else
             PyQt_PyObject pyobj_wrapper = qVariantValue<PyQt_PyObject>(var);
+#endif
 
             if (!pyobj_wrapper.pyobject)
             {
@@ -1488,12 +1517,14 @@ PyObject *Chimera::toPyObject(void *cpp) const
                 sipType_QObject, 0);
         break;
 
+#if QT_VERSION < 0x050000
     case QMetaType::QWidgetStar:
         if (sipType_QWidget)
             py = sipConvertFromType(*reinterpret_cast<void **>(cpp),
                     sipType_QWidget, 0);
 
         break;
+#endif
 
     case QMetaType::QVariantList:
         {
@@ -1594,7 +1625,11 @@ PyObject *Chimera::toPyObject(void *cpp) const
             else
             {
                 // Make a copy as it is a value type.
+#if QT_VERSION >= 0x050000
+                void *copy = QMetaType::create(_metatype, cpp);
+#else
                 void *copy = QMetaType::construct(_metatype, cpp);
+#endif
 
                 py = sipConvertFromNewType(copy, _type, 0);
 
