@@ -1,6 +1,6 @@
 // This contains the implementation of the pyqtBoundSignal type.
 //
-// Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -46,6 +46,7 @@ static PyObject *pyqtBoundSignal_call(PyObject *self, PyObject *args,
 static void pyqtBoundSignal_dealloc(PyObject *self);
 static PyObject *pyqtBoundSignal_repr(PyObject *self);
 static PyObject *pyqtBoundSignal_get_doc(PyObject *self, void *);
+static PyObject *pyqtBoundSignal_get_signal(PyObject *self, void *);
 static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
         PyObject *kwd_args);
 static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args);
@@ -58,7 +59,7 @@ static PyObject *connect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         const char *slot, Qt::ConnectionType type);
 static PyObject *disconnect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         const char *slot);
-static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
+static QObject *get_receiver(qpycore_pyqtBoundSignal *bs, PyObject *slot_obj,
         QByteArray &name);
 static QByteArray slot_signature_from_decorations(Chimera::Signature *signal,
         PyObject *decorations, int nr_args);
@@ -87,6 +88,9 @@ PyDoc_STRVAR(pyqtBoundSignal_emit_doc,
 "*args are the values that will be passed as arguments to all connected\n"
 "slots.");
 
+PyDoc_STRVAR(pyqtBoundSignal_signal_doc,
+"The signature of the signal that would be returned by SIGNAL()");
+
 
 // Define the methods.
 static PyMethodDef pyqtBoundSignal_methods[] = {
@@ -112,6 +116,8 @@ static PyMappingMethods pyqtBoundSignal_as_mapping = {
 // The getters/setters.
 static PyGetSetDef pyqtBoundSignal_getsets[] = {
     {(char *)"__doc__", pyqtBoundSignal_get_doc, NULL, NULL, NULL},
+    {(char *)"signal", pyqtBoundSignal_get_signal, NULL,
+            (char *)pyqtBoundSignal_signal_doc, NULL},
     {NULL, NULL, NULL, NULL, NULL}
 };
 
@@ -175,7 +181,39 @@ static PyObject *pyqtBoundSignal_get_doc(PyObject *self, void *)
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
-    return qpycore_get_signal_doc(bs->unbound_signal);
+    const char *docstring = bs->unbound_signal->docstring;
+
+    if (!docstring)
+    {
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+
+    if (*docstring == '\1')
+        ++docstring;
+
+    return
+#if PY_MAJOR_VERSION >= 3
+        PyUnicode_FromString
+#else
+        PyString_FromString
+#endif
+            (docstring);
+}
+
+
+// The __signal__ getter.
+static PyObject *pyqtBoundSignal_get_signal(PyObject *self, void *)
+{
+    qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
+
+    return
+#if PY_MAJOR_VERSION >= 3
+        PyUnicode_FromString
+#else
+        PyString_FromString
+#endif
+            (bs->unbound_signal->signature->signature.constData());
 }
 
 
@@ -184,7 +222,7 @@ static PyObject *pyqtBoundSignal_repr(PyObject *self)
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
-    QByteArray name = Chimera::Signature::name(bs->bound_overload->signature);
+    QByteArray name = bs->unbound_signal->signature->name();
 
     return
 #if PY_MAJOR_VERSION >= 3
@@ -213,26 +251,25 @@ static void pyqtBoundSignal_dealloc(PyObject *self)
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
-    Py_XDECREF(bs->unbound_signal);
+    Py_XDECREF((PyObject *)bs->unbound_signal);
 
     Py_TYPE(self)->tp_free(self);
 }
 
 
 // Create a bound signal.
-PyObject *qpycore_pyqtBoundSignal_New(PyObject *unbound_signal,
-        PyObject *bound_pyobject, QObject *bound_qobject, int signal_index)
+PyObject *qpycore_pyqtBoundSignal_New(qpycore_pyqtSignal *unbound_signal,
+        PyObject *bound_pyobject, QObject *bound_qobject)
 {
-    qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)PyType_GenericAlloc(&qpycore_pyqtBoundSignal_Type, 0);
+    qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)PyType_GenericNew(&qpycore_pyqtBoundSignal_Type, 0, 0);
 
     if (bs)
     {
-        Py_INCREF(unbound_signal);
+        Py_INCREF((PyObject *)unbound_signal);
         bs->unbound_signal = unbound_signal;
 
         bs->bound_pyobject = bound_pyobject;
         bs->bound_qobject = bound_qobject;
-        bs->bound_overload = ((qpycore_pyqtSignal *)unbound_signal)->overloads->at(signal_index);
     }
 
     return (PyObject *)bs;
@@ -245,15 +282,15 @@ static PyObject *pyqtBoundSignal_mp_subscript(PyObject *self,
 {
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
-    int idx = qpycore_signal_index((qpycore_pyqtSignal *)bs->unbound_signal,
-            subscript, "a bound signal type argument");
+    qpycore_pyqtSignal *ps = qpycore_find_signal(bs->unbound_signal, subscript,
+            "a bound signal type argument");
 
-    if (idx < 0)
+    if (!ps)
         return 0;
 
     // Create a new bound signal.
-    return qpycore_pyqtBoundSignal_New(bs->unbound_signal, bs->bound_pyobject,
-            bs->bound_qobject, idx);
+    return qpycore_pyqtBoundSignal_New(ps, bs->bound_pyobject,
+            bs->bound_qobject);
 }
 
 
@@ -311,7 +348,8 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
         }
 
         return connect(bs, slot_bs->bound_qobject,
-                slot_bs->bound_overload->signature.constData(), type);
+                slot_bs->unbound_signal->signature->signature.constData(),
+                type);
     }
 
     // Make sure the slot is callable.
@@ -327,7 +365,7 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
     // See if the slot can be used directly (ie. it wraps a Qt slot) or if it
     // needs a proxy.
     QByteArray rx_name;
-    QObject *rx_qobj = get_receiver(bs->bound_overload, slot_obj, rx_name);
+    QObject *rx_qobj = get_receiver(bs, slot_obj, rx_name);
 
     if (PyErr_Occurred())
         return 0;
@@ -343,7 +381,7 @@ static PyObject *pyqtBoundSignal_connect(PyObject *self, PyObject *args,
 
     proxy = new PyQtProxy(bs, slot_obj, &member);
 
-    if (proxy->real_slot.signature)
+    if (proxy->metaObject())
     {
         if (rx_qobj)
             proxy->moveToThread(rx_qobj->thread());
@@ -368,10 +406,11 @@ static PyObject *connect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         const char *slot, Qt::ConnectionType type)
 {
     bool ok;
+    Chimera::Signature *signature = bs->unbound_signal->signature;
 
     Py_BEGIN_ALLOW_THREADS
-    ok = QObject::connect(bs->bound_qobject,
-            bs->bound_overload->signature.constData(), qrx, slot, type);
+    ok = QObject::connect(bs->bound_qobject, signature->signature.constData(),
+            qrx, slot, type);
     Py_END_ALLOW_THREADS
 
     if (!ok)
@@ -379,8 +418,7 @@ static PyObject *connect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         QByteArray rx_name = Chimera::Signature::name(slot);
 
         PyErr_Format(PyExc_TypeError, "connect() failed between %s and %s()",
-                bs->bound_overload->py_signature.constData(),
-                rx_name.constData() + 1);
+                signature->py_signature.constData(), rx_name.constData() + 1);
 
         return 0;
     }
@@ -396,6 +434,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
     qpycore_pyqtBoundSignal *bs = (qpycore_pyqtBoundSignal *)self;
 
     PyObject *slot_obj = 0, *res_obj;
+    Chimera::Signature *signature = bs->unbound_signal->signature;
 
 #if PY_VERSION_HEX >= 0x02050000
     if (!PyArg_ParseTuple(args, "|O:disconnect", &slot_obj))
@@ -410,7 +449,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
         res_obj = disconnect(bs, 0, 0);
 
         PyQtProxy::deleteSlotProxies(bs->bound_qobject,
-                bs->bound_overload->signature.constData());
+                signature->signature.constData());
 
         return res_obj;
     }
@@ -421,7 +460,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
         qpycore_pyqtBoundSignal *slot_bs = (qpycore_pyqtBoundSignal *)slot_obj;
 
         return disconnect(bs, slot_bs->bound_qobject,
-                slot_bs->bound_overload->signature.constData());
+                slot_bs->unbound_signal->signature->signature.constData());
     }
 
     if (!PyCallable_Check(slot_obj))
@@ -436,7 +475,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
     // See if the slot has been used directly (ie. it wraps a Qt slot) or if it
     // has a proxy.
     QByteArray rx_name;
-    QObject *rx_qobj = get_receiver(bs->bound_overload, slot_obj, rx_name);
+    QObject *rx_qobj = get_receiver(bs, slot_obj, rx_name);
 
     if (PyErr_Occurred())
         return 0;
@@ -446,7 +485,7 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
 
     const char *member;
     PyQtProxy *proxy = PyQtProxy::findSlotProxy(bs->bound_qobject,
-            bs->bound_overload->signature.constData(), slot_obj, 0, &member);
+            signature->signature.constData(), slot_obj, 0, &member);
 
     if (!proxy)
     {
@@ -468,16 +507,17 @@ static PyObject *pyqtBoundSignal_disconnect(PyObject *self, PyObject *args)
 static PyObject *disconnect(qpycore_pyqtBoundSignal *bs, QObject *qrx,
         const char *slot)
 {
+    Chimera::Signature *signature = bs->unbound_signal->signature;
     bool ok;
 
     Py_BEGIN_ALLOW_THREADS
     ok = QObject::disconnect(bs->bound_qobject,
-            bs->bound_overload->signature.constData(), qrx, slot);
+            signature->signature.constData(), qrx, slot);
     Py_END_ALLOW_THREADS
 
     if (!ok)
     {
-        QByteArray tx_name = bs->bound_overload->name();
+        QByteArray tx_name = signature->name();
 
         if (slot)
         {
@@ -509,11 +549,31 @@ static PyObject *pyqtBoundSignal_emit(PyObject *self, PyObject *args)
 
     if (!bs->bound_qobject->signalsBlocked())
     {
-        int signal_index = bs->bound_qobject->metaObject()->indexOfSignal(bs->bound_overload->signature.constData() + 1);
+        Chimera::Signature *signature = bs->unbound_signal->signature;
+        int mo_index = bs->bound_qobject->metaObject()->indexOfSignal(signature->signature.constData() + 1);
 
-        Q_ASSERT(signal_index >= 0);
+        if (mo_index < 0)
+        {
+            PyErr_Format(PyExc_AttributeError,
+                    "signal was not defined in the first super-class of class '%s'",
+                    Py_TYPE(bs->bound_pyobject)->tp_name);
+            return 0;
+        }
 
-        if (!qpycore_emit(bs->bound_qobject, signal_index, bs->bound_overload, args))
+        // Use the docstring if there is one and it is auto-generated.
+        const char *docstring = bs->unbound_signal->docstring;
+
+        if (!docstring || *docstring != '\1')
+        {
+            docstring = signature->py_signature.constData();
+        }
+        else
+        {
+            // Skip the auto-generated marker.
+            ++docstring;
+        }
+
+        if (!qpycore_emit(bs->bound_qobject, mo_index, signature, docstring, args))
             return 0;
     }
 
@@ -525,12 +585,13 @@ static PyObject *pyqtBoundSignal_emit(PyObject *self, PyObject *args)
 // Get the receiver QObject from the slot (if there is one) and its signature
 // (if it wraps a Qt slot).  A Python exception will be raised if there was an
 // error.
-static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
+static QObject *get_receiver(qpycore_pyqtBoundSignal *bs, PyObject *slot_obj,
         QByteArray &name)
 {
     PyObject *rx_self, *decorations;
     QByteArray rx_name;
     bool try_qt_slot;
+    Chimera::Signature *signature = bs->unbound_signal->signature;
 
     decorations = 0;
 
@@ -599,7 +660,7 @@ static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
     // of them) then use it.  Otherwise we will fallback to using a proxy.
     if (try_qt_slot)
     {
-        for (int ol = overload->parsed_arguments.count(); ol >= 0; --ol)
+        for (int ol = signature->parsed_arguments.count(); ol >= 0; --ol)
         {
             // If there are decorations then we compare the signal's signature
             // against them so that we distinguish between Python types that
@@ -607,10 +668,10 @@ static QObject *get_receiver(Chimera::Signature *overload, PyObject *slot_obj,
             // distinction.  If there are no decorations then let Qt determine
             // if a slot is available.
             if (decorations)
-                name = slot_signature_from_decorations(overload, decorations,
+                name = slot_signature_from_decorations(signature, decorations,
                         ol);
             else
-                name = slot_signature_from_metaobject(overload,
+                name = slot_signature_from_metaobject(signature,
                         rx_qobj->metaObject(), rx_name, ol);
 
             if (!name.isEmpty())
@@ -744,14 +805,14 @@ const char *qpycore_pyqtsignal_get_parts(PyObject *sig_obj, QObject **qtx)
         if (qtx)
             *qtx = bs->bound_qobject;
 
-        return bs->bound_overload->signature.constData();
+        return bs->unbound_signal->signature->signature.constData();
     }
 
     if (PyObject_TypeCheck(sig_obj, &qpycore_pyqtSignal_Type) && !qtx)
     {
         qpycore_pyqtSignal *ps = (qpycore_pyqtSignal *)sig_obj;
 
-        return ps->overloads->at(ps->master_index)->signature.constData();
+        return ps->signature->signature.constData();
     }
 
     return 0;
