@@ -1,6 +1,6 @@
 // This implements the helpers for QObject.
 //
-// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt.
 // 
@@ -16,13 +16,8 @@
 // GPL Exception version 1.1, which can be found in the file
 // GPL_EXCEPTION.txt in this package.
 // 
-// Please review the following information to ensure GNU General
-// Public Licensing requirements will be met:
-// http://trolltech.com/products/qt/licenses/licensing/opensource/. If
-// you are unsure which license is appropriate for your use, please
-// review the following information:
-// http://trolltech.com/products/qt/licenses/licensing/licensingoverview
-// or contact the sales department at sales@riverbankcomputing.com.
+// If you are unsure which license is appropriate for your use, please
+// contact the sales department at sales@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -59,7 +54,7 @@ const QMetaObject *qpycore_qobject_metaobject(sipSimpleWrapper *pySelf,
 {
     // Return the dynamic meta-object if there is one.
     if (pySelf && ((pyqtWrapperType *)Py_TYPE(pySelf))->metaobject)
-        return &((pyqtWrapperType *)Py_TYPE(pySelf))->metaobject->mo;
+        return QPYCORE_QMETAOBJECT(((pyqtWrapperType *)Py_TYPE(pySelf))->metaobject);
 
     // Fall back to the static Qt meta-object.
     return reinterpret_cast<const QMetaObject *>(((pyqt4ClassTypeDef *)base)->qt4_static_metaobject);
@@ -109,7 +104,7 @@ static int qt_metacall_worker(sipSimpleWrapper *pySelf, PyTypeObject *pytype,
                 QObject *qthis = reinterpret_cast<QObject *>(sipGetCppPtr(pySelf, sipType_QObject));
 
                 Py_BEGIN_ALLOW_THREADS
-                QMetaObject::activate(qthis, &qo->mo, _id, _a);
+                QMetaObject::activate(qthis, QPYCORE_QMETAOBJECT(qo), _id, _a);
                 Py_END_ALLOW_THREADS
             }
             else
@@ -158,11 +153,14 @@ static int qt_metacall_worker(sipSimpleWrapper *pySelf, PyTypeObject *pytype,
 
                     if (var)
                     {
-                        // This tells QMetaProperty::read() to use the new
-                        // contents of the QVariant it provided.
-                        _a[1] = 0;
-
                         ok = prop->pyqtprop_parsed_type->fromPyObject(py, var);
+
+                        // Make sure that _a[0] still points to the QVariant
+                        // data (whose address we may have just changed) so
+                        // that QMetaProperty::read() doesn't try to create a 
+                        // new QVariant.
+                        if (ok)
+                            _a[0] = var->data();
                     }
                     else
                     {
@@ -305,11 +303,29 @@ PyObject *qpycore_qobject_staticmetaobject(PyObject *type_obj)
     const QMetaObject *mo;
 
     if (pyqt_wt->metaobject)
+    {
         // It's a sub-type of a wrapped type.
-        mo = &pyqt_wt->metaobject->mo;
+        mo = QPYCORE_QMETAOBJECT(pyqt_wt->metaobject);
+    }
     else
+    {
         // It's a wrapped type.
-        mo = reinterpret_cast<const QMetaObject *>(((pyqt4ClassTypeDef *)((sipWrapperType *)pyqt_wt)->type)->qt4_static_metaobject);
+        pyqt4ClassTypeDef *p4ctd = (pyqt4ClassTypeDef *)((sipWrapperType *)pyqt_wt)->type;
+
+        if (!p4ctd)
+        {
+            /*
+             * This is a side effect of a wrapped class not being fully ready
+             * until sip's meta-class's __init__() has run (rather than after
+             * its __new__() method as might be expected).
+             */
+            PyErr_SetString(PyExc_AttributeError,
+                    "staticMetaObject isn't available until the meta-class's __init__ returns");
+            return 0;
+        }
+
+        mo = reinterpret_cast<const QMetaObject *>(p4ctd->qt4_static_metaobject);
+    }
 
     return sipConvertFromType(const_cast<QMetaObject *>(mo), sipType_QMetaObject, 0);
 }
@@ -322,7 +338,7 @@ QObject *qpycore_qobject_sender(QObject *obj)
         return obj;
 
     // See if it is a short-circuit signal proxy.
-    PyQtShortcircuitSignalProxy *ssp = qobject_cast<PyQtShortcircuitSignalProxy *>(PyQtProxy::last_sender);
+    PyQtShortcircuitSignalProxy *ssp = PyQtShortcircuitSignalProxy::shortcircuitSignal(PyQtProxy::last_sender);
 
     if (ssp)
         return ssp->parent();
@@ -352,7 +368,7 @@ int qpycore_qobject_receivers(QObject *obj, const char *signal, int nr)
         return nr;
 
     // See if it is a short-circuit signal proxy.
-    PyQtShortcircuitSignalProxy *ssp = qobject_cast<PyQtShortcircuitSignalProxy *>(qtx);
+    PyQtShortcircuitSignalProxy *ssp = PyQtShortcircuitSignalProxy::shortcircuitSignal(qtx);
 
     if (ssp)
         return ssp->getReceivers(signal);
